#!/usr/bin/env python

# Used to plot CDF of the FB transmission delay of RTP/AVPF

import sys, commands
import re
#import matplotlib
#matplotlib.use('PS')
from matplotlib.pylab import *

eventPattern = re.compile(r"^(\d+)\.(\d+): *<\[(\d+)\]>")

class DecLog:
    
    def __init__ (self, filename):
        try:
            self.file =  file(filename, 'r')
        except IOError, (errno, strerror):
            print "Error opening file: %s" % filename
            print "I/O error(%s): %s" % (errno, strerror)
            raise

    def getNextEvent (self):
        event = None
        line = self.file.readline()
        while(line != ''):
            try:
                sec, ms, event = eventPattern.match(line).groups()
                event = Event(int(event), int(sec), int(ms))
                break
            except:
                line = self.file.readline()
        return event

    def free (self):
        self.file.close()
        

class Event:
    "Store info about an event in RTCP receiver"
    sec = 0
    ms = 0
    time  = 0                              #milliseconds
    type = 0

    def __init__(self, type, sec, ms):
        self.type = type
        self.sec = sec
        self.ms = ms
        self.time = sec * 1000 + ms

    def __str__ (self):
        return "Type: %2d\t Time: %d.%03d" % (self.type, self.sec, self.ms)


def findLogFiles(clip, scheme, rate, delay, loss):
    cmd = "find . -name '*-dec.log'"
    if clip != None:
        cmd += "| grep %s" % clip
    if scheme != None:
        cmd += "| grep %s" % scheme
    if rate != None:
        cmd += "| grep %03dkbps" % rate
    if delay != None:
        cmd += "| grep %03dms" % delay
    if loss != None:
        cmd += "| grep %02dplr" % loss

    res = commands.getoutput(cmd)
    files = []
    for file in res.split("\n"):
        files.append(file)
    return files

def getFBTime (clip, scheme, rate, delay, lossrate):

    logfiles = findLogFiles(clip, scheme, rate, delay, lossrate)    
    fbtime = []

    for file in logfiles:
        losstime = []
        
        logfile = DecLog(file)
        #rr_got_fb = False
        
        event = logfile.getNextEvent()
        while (event != None ):
            events = {"RTCP_LOSS_DETECTED": 1, \
                          "RTCP_SCHEDULE_FB": 2,\
                          "RTCP_ADD_FB_TO_FB": 3, \
                          "RTCP_ADD_FB_TO_RR": 4, \
                          "RTCP_LOSS_IGNORED": 5, \
                          "RTCP_SEND_FB": 6, \
                          "RTCP_SEND_RR": 7}

            if (event.type == events["RTCP_LOSS_DETECTED"]):
                losstime.append(event.time)

            elif (event.type == events["RTCP_SCHEDULE_FB"]):
                pass

            elif (event.type == events["RTCP_ADD_FB_TO_FB"]):
                pass

            elif (event.type == events["RTCP_ADD_FB_TO_RR"]):
                pass

            elif (event.type == events["RTCP_LOSS_IGNORED"]):
                losstime.pop()


            elif (event.type == events["RTCP_SEND_FB"]):

                if (len(losstime) != 0):
                    for loss in losstime:
                        fbtime.append(event.time - loss)
                    losstime = []

            elif (event.type == events["RTCP_SEND_RR"]):

                if (len(losstime) != 0):
                    for loss in losstime:
                        fbtime.append(event.time - loss)
                    losstime = []

            else:
                print "Error: \"Event " + str(event) + "\" unsupported"

            event = logfile.getNextEvent()

        logfile.free()

    return fbtime


def main():

    fbtime1_384 = getFBTime("conv", "fbir", 384, None, 1)

    fbtime3_384 = getFBTime("conv", "fbir", 384, None, 3)
    fbtime5_384 = getFBTime("conv", "fbir", 384, None, 5)

    fbtime1_144 = getFBTime("conv", "fbir", 144, None, 1)
    fbtime3_144 = getFBTime("conv", "fbir", 144, None, 3)
    fbtime5_144 = getFBTime("conv", "fbir", 144, None, 5)

    fbtime1_064 = getFBTime("conv", "fbir", 64, None, 1)
    fbtime3_064 = getFBTime("conv", "fbir", 64, None, 3)
    fbtime5_064 = getFBTime("conv", "fbir", 64, None, 5)


    #############  
    # Plot histograms
    min_val = 0
    max_val = 800
#     max_val = max ( [ max(fbtime1_384),  \
#                       max(fbtime3_384),  \
#                       max(fbtime5_384),  \
#                       max(fbtime1_144),  \
#                       max(fbtime3_144),  \
#                       max(fbtime5_144),  \
#                       max(fbtime1_064),   \
#                       max(fbtime3_064),   \
#                       max(fbtime5_064)] )

    # Plot CDFs
    figno = 1
    axis_cdf = [min_val, max_val, 0, 1]
    
    figure(figno)
    hold(True)
    grid(True)
    plot(sort(fbtime3_384), linspace(0, 1, len(fbtime3_384)), '-.r', label='384 kbit/s')
    plot(sort(fbtime3_144), linspace(0, 1, len(fbtime3_144)), '-b', label='144 kbit/s')
    plot(sort(fbtime3_064), linspace(0, 1, len(fbtime3_064)), '--g', label='64 kbit/s')
    axis(axis_cdf)
    xlabel("Delay (ms)")
    ylabel("Cumulative probability")
    title("CDF: FB transmission delay, 3% loss")
    legend(loc='lower right')
    savefig("fb-cdf-03plr.eps")
    figno = figno + 1


    figure(figno)
    hold(True)
    grid(True)
    plot(sort(fbtime1_064), linspace(0, 1, len(fbtime1_064)), '-.r', label='1% loss')
    plot(sort(fbtime3_064), linspace(0, 1, len(fbtime3_064)), '-b', label='3% loss')
    plot(sort(fbtime5_064), linspace(0, 1, len(fbtime5_064)), '--g', label='5% loss')
    axis(axis_cdf)
    xlabel("Delay (ms)")
    ylabel("Cumulative probability")
    title("CDF: FB transmission delay, 64 kbit/s")
    legend(loc='lower right')
    savefig("fb-cdf-064kbps.eps")
    figno = figno + 1

    return


#     # 384 kbps
#     figure(figno)
#     plot(sort(fbtime1_384), linspace(0, 1, len(fbtime1_384)))
#     axis(axis_cdf)
#     xlabel("CDF: FB transmission delay (ms)")
#     title("384 kbps, 1% loss")
#     savefig("fbtime-384-01plr-cdf.eps")
#     figno = figno + 1
    
#     figure(figno)
#     plot(sort(fbtime3_384), linspace(0, 1, len(fbtime3_384)))
#     axis(axis_cdf)
#     xlabel("CDF: FB transmission delay (ms)")
#     title("384 kbps, 3% loss")
#     savefig("fbtime-384-03plr-cdf.eps")
#     figno = figno + 1

#     figure(figno)
#     plot(sort(fbtime5_384), linspace(0, 1, len(fbtime5_384)))
#     axis(axis_cdf)
#     xlabel("CDF: FB transmission delay (ms)")
#     title("384 kbps, 5% loss")
#     savefig("fbtime-384-05plr-cdf.eps")
#     figno = figno + 1


#     # 144 kbps
#     figure(figno)
#     plot(sort(fbtime1_144), linspace(0, 1, len(fbtime1_144)))
#     axis(axis_cdf)
#     xlabel("CDF: FB transmission delay (ms)")
#     title("144 kbps, 1% loss")
#     savefig("fbtime-144-01plr-cdf.eps")
#     figno = figno + 1
    
#     figure(figno)
#     plot(sort(fbtime3_144), linspace(0, 1, len(fbtime3_144)))
#     axis(axis_cdf)
#     xlabel("CDF: FB transmission delay (ms)")
#     title("144 kbps, 3% loss")
#     savefig("fbtime-144-03plr-cdf.eps")
#     figno = figno + 1

#     figure(figno)
#     plot(sort(fbtime5_144), linspace(0, 1, len(fbtime5_144)))
#     axis(axis_cdf)
#     xlabel("CDF: FB transmission delay (ms)")
#     title("144 kbps, 5% loss")
#     savefig("fbtime-144-05plr-cdf.eps")
#     figno = figno + 1


#     # 064 kbps
#     figure(figno)
#     plot(sort(fbtime1_064), linspace(0, 1, len(fbtime1_064)))
#     axis(axis_cdf)
#     xlabel("CDF: FB transmission delay (ms)")
#     title("064 kbps, 1% loss")
#     savefig("fbtime-064-01plr-cdf.eps")
#     figno = figno + 1
    
#     figure(figno)
#     plot(sort(fbtime3_064), linspace(0, 1, len(fbtime3_064)))
#     axis(axis_cdf)
#     xlabel("CDF: FB transmission delay (ms)")
#     title("064 kbps, 3% loss")
#     savefig("fbtime-064-03plr-cdf.eps")
#     figno = figno + 1

#     figure(figno)
#     plot(sort(fbtime5_064), linspace(0, 1, len(fbtime5_064)))
#     axis(axis_cdf)
#     xlabel("CDF: FB transmission delay (ms)")
#     title("064 kbps, 5% loss")
#     savefig("fbtime-064-05plr-cdf.eps")
#     figno = figno + 1




    ###############
    # Draw histograms
    # Todo: make figures have same y axis

    width = 25
    bins = range(min_val, max_val, width)

    # 384 kbps
    figure(figno)
    hist(fbtime1_384, bins, normed=1)
    xlabel("Time until feedback sent (ms)")
    title("384 kbps, 1% loss")
    savefig("fbtime-384-01plr.eps")
    figno = figno + 1
    
    figure(figno)
    hist(fbtime3_384, bins, normed=1)
    xlabel("Time until feedback sent (ms)")
    title("384 kbps, 3% loss")
    savefig("fbtime-384-03plr.eps")
    figno = figno + 1

    figure(figno)
    hist(fbtime5_384, bins, normed=1)
    xlabel("Time until feedback sent (ms)")
    title("384 kbps, 5% loss")
    savefig("fbtime-384-05plr.eps")
    figno = figno + 1


    # 144 kbps
    figure(figno)
    hist(fbtime1_144, bins, normed=1)
    xlabel("Time until feedback sent (ms)")
    title("144 kbps, 1% loss")
    savefig("fbtime-144-01plr.eps")
    figno = figno + 1
    
    figure(figno)
    hist(fbtime3_144, bins, normed=1)
    xlabel("Time until feedback sent (ms)")
    title("144 kbps, 3% loss")
    savefig("fbtime-144-03plr.eps")
    figno = figno + 1

    figure(figno)
    hist(fbtime5_144, bins, normed=1)
    xlabel("Time until feedback sent (ms)")
    title("144 kbps, 5% loss")
    savefig("fbtime-144-05plr.eps")
    figno = figno + 1


    # 064 kbps
    figure(figno)
    hist(fbtime1_064, bins, normed=1)
    xlabel("Time until feedback sent (ms)")
    title("064 kbps, 1% loss")
    savefig("fbtime-064-01plr.eps")
    figno = figno + 1
    
    figure(figno)
    hist(fbtime3_064, bins, normed=1)
    xlabel("Time until feedback sent (ms)")
    title("064 kbps, 3% loss")
    savefig("fbtime-064-03plr.eps")
    figno = figno + 1

    figure(figno)
    hist(fbtime5_064, bins, normed=1)
    xlabel("Time until feedback sent (ms)")
    title("064 kbps, 5% loss")
    savefig("fbtime-064-05plr.eps")
    figno = figno + 1

    #show()

if __name__ == "__main__":
    main()
