#!/bin/bash

# Used for listing the video files that have changed since last execution of 
# the script.

# Call the script like this: "./new-or-changed '.' '.*txt'"
# '.' is the directory to search in.
# '.*txt' is the regular expression to search for.

STATUSFILE=new-or-changed.dat
STATUSFILE_TMP=new-or-changed.dat.tmp
STATUSFILE_BAK=new-or-changed.dat.bak

ROOTDIR="$1" #'.'
SEARCH_REGEX="$2" #'.*txt'

function new_or_changed {

    # Remove existing temporary file (but it should not be there)
    rm -f $STATUSFILE_TMP

    # Create a new dummy md5sum file if it does not exist
    if [ ! -e $STATUSFILE ]; then
        touch $STATUSFILE
    fi

    # Backup md5-file
    cp -f $STATUSFILE $STATUSFILE_BAK

    # Generate md5 sums of images
    for file in `find $1 -iregex "$2" | sort`; do
        ls --full-time $file | awk '{print $9 "\t" $5 "\t" $6 "\t" $7 "\t" $8}' >> $STATUSFILE_TMP
    done
    
    # If we got hits on regex, tmp file exists and we do a diff
    if [ -e $STATUSFILE_TMP ]; then
	diff $STATUSFILE $STATUSFILE_TMP | grep -e '^>' | awk '{print $2}'
	mv -f $STATUSFILE_TMP $STATUSFILE
    fi
    
}

new_or_changed "$ROOTDIR" "$SEARCH_REGEX"

#for file in $(new_or_changed "$ROOTDIR" "$SEARCH_REGEX"); do
#    echo "Do something with file $file"
#done

