#!/usr/bin/env python

# Used for decoding all received video, calculate the PSNRs and find the 
# average PSNR.

import os
import commands
import re

foreyuv = "/home/stianse/master/thesis/code/jm/bin/foreman_qcif_10fps_4000f.yuv"
convyuv = "/home/stianse/master/thesis/code/jm/bin/conversation_qcif_10fps_4000f.yuv"
soccyuv = "/home/stianse/master/thesis/code/jm/bin/soccer_qcif_10fps_4000f.yuv"
psnrcmd = "psnr 176 144 420 "
deccmd = "ldecod.exe "
newsimscmd = "./new-or-changed.sh '.' '.*\.264'"


cfg_part2 = """test_rec_rtp.yuv             ........Ref sequence (for SNR)
1                        ........Write 4:2:0 chroma components for monochrome streams
1                        ........NAL mode (0=Annex B, 1: RTP packets)
62002                    ........Receiving port for RTP stream
10.0.0.3                 ........Host for sending RTCP feedback messages to
62003                    ........Port for sending RTCP feedback messages to
144
0                        ........SNR computation offset
2                        ........Poc Scale (1 or 2)
500000                   ........Rate_Decoder
104000                   ........B_decoder
73000                    ........F_decoder
leakybucketparam.cfg     ........LeakyBucket Params
2                        ........Err Concealment(0:Off,1:Frame Copy,2:Motion Copy)
2                        ........Reference POC gap (2: IPP (Default), 4: IbP / IpP)
2                        ........POC gap (2: IPP /IbP/IpP (Default), 4: IPP with frame skip = 1 etc.) 

This is a file containing input parameters to the JVT H.264/AVC decoder.
The text line following each parameter is discarded by the decoder.
"""

def decode_and_psnr_video(dir, file):
  cwd = os.getcwd()
  os.chdir(dir)
  
  yuv = file.replace(".264", ".yuv")
  cfg = file + "\n" + yuv + "\n" + cfg_part2
  
  # Write config file temporarily
  cfgfn = "decoder.cfg"
  cfgfile = open(cfgfn,"w")
  cfgfile.writelines(cfg)
  cfgfile.close()
  
  # Decode the video
  print "Decoding '%s' in dir '%s'" % (file, dir)
  rc,out = commands.getstatusoutput(deccmd + cfgfn)
  if (rc != 0):
    print "ERROR decoding video " + file
    print deccmd + " returned with code " + str(rc)
#    print "Output:\n" + out

  # Find psnr and store to file
  print "Finding PSNR for '%s' in dir '%s'" % (yuv, dir)
  psnrfile = file.replace(".264", ".psnr")
  if dir.find("conv") != -1:
    origyuv = convyuv
  elif dir.find("socc") != -1:
    origyuv = soccyuv
  elif dir.find("fore") != -1:
    origyuv = foreyuv
  else:
    print "Error: Did not understand what original yuv file to use"
    origyuv = "unknown"
  
  rc,out = commands.getstatusoutput(psnrcmd + origyuv + " " + yuv + " > " + psnrfile)
  if (rc != 0):
    print "ERROR finding PSNR for " + yuv
#    print out

  # Clean up
  delfiles = ["data*", "*.cfg", "*.dat", "*.dec", "*.yuv"]
  commands.getoutput("rm -f " + " ".join(delfiles))
  os.chdir(cwd)

def psnr_parse_dir(dir):
  cwd = os.getcwd()
  os.chdir(dir)
  psnrfiles = commands.getoutput("find . -name '*.psnr'")
  means = []
  for psnrfile in psnrfiles.split("\n"):
    if psnrfile == '':
      continue
    mean_tmp = commands.getoutput("tail -n 1 " + psnrfile + " | awk '{print $5 \" \" $7}'").split(" ")[0]
    means.append(float(mean_tmp))

  # Calculate average PSNR
  mean = sum(means) / len(means)
  diff = []
  for onemean in means:
    diff.append((onemean - mean)**2)
  if (len(diff) > 1):
    var = sum(diff) / (len(diff)-1)
  else:
    var = 0.0

  # Write to file
  commands.getoutput("echo mean: " + str(mean) + "\t var: " + str(var) + " > total-psnr.txt")

  # Print on stdout
  dirsplit = dir.split("/")
  print "%s-%s-%s-%s: %s : mean = %.1f, var = %.2f" % (dirsplit[1], dirsplit[2], dirsplit[3], dirsplit[4], dirsplit[5], mean, var)

  os.chdir(cwd)

def main():

  #  allvids = commands.getoutput("find . -name '*.264' | sort")
  #  for vid in allvids.split("\n"):

  # only decode and calculate psnr for new simulations (time consuming process)
  print "Finding new and changed simulations"
  newvids = commands.getoutput(newsimscmd)
  for vid in newvids.split("\n"):
    if vid == '':
      continue
    dir = "/".join(vid.split("/")[0:-1])
    file = vid.split("/")[-1]
    decode_and_psnr_video(dir, file)

  # parse psnr-files and calculate average psnrs
  print "Calculating total PSNRs"
  dirs = commands.getoutput("find . -name '*.264' | sed -e 's/\/[^\/]*$//' | sort | uniq")
  for dir in dirs.split("\n"):
    psnr_parse_dir(dir)
  
if __name__ == "__main__":
  main()
