#!/usr/bin/env python

# Used to plot all measured average Y-PSNR with respect to loss rate

from matplotlib.pylab import *
import commands, re

class DataSet2D:

    def __init__ (self):
        self.data = [[],[]]

    def addData (self, x, y):
        self.data[0].append(x)
        self.data[1].append(y)

    def getDataX (self):
        return self.data[0]

    def getDataY (self):
        return self.data[1]


def calcKey (clip, scheme, rate, delay):
    return "%s_%s_%dkbps_%dms" % (clip.lower(), scheme.lower(), rate, delay)


def main():

    data = {}
    psnrfiles = commands.getoutput("find . -name '*total-psnr.txt' | sort")
    
    # Get all PSNR values from file
    for filepath in psnrfiles.split("\n"):
        clip = ""
        scheme = ""
        rate = -1
        delay = -1
        loss = -1
    

        # Parse filename to get parameter set
        if filepath.find("conv") != -1:
            clip = "Conversation"
        elif filepath.find("socc") != -1:
            clip = "Soccer"
        elif filepath.find("fore") != -1:
            clip = "Foreman"
        else:
            print "Unknow clip for file: '%s'" % filepath

        if filepath.find("fbir") != -1:
            scheme = "FBIR"
        elif filepath.find("iplr") != -1:
            scheme = "IPLR"
        else:
            print "Unknow scheme for file: '%s'" % filepath
    
        m = re.search(r"(\d+)kbps", filepath)
        if m != None:
            rate = int(m.groups()[0])
        else:
            print "Unknow bitrate for file: '%s'" % filepath
            
        m = re.search(r"(\d+)ms", filepath)
        if m != None:
            delay = int(m.groups()[0])
        else:
            print "Unknow delay for file: '%s'" % filepath
            
        m = re.search(r"(\d+)plr", filepath)
        if m != None:
            loss = int(m.groups()[0])
        else:
            print "Unknow lossrate for file: '%s'" % filepath
        
        # Read and update PSNR data
        psnr = float(commands.getoutput("head -n 1 " + filepath).split(" ")[1])

        key = calcKey(clip, scheme, rate, delay)
        if data.has_key(key) is False:
            data[key] = DataSet2D()
        data[key].addData(loss, psnr)


    ################################################################################################
    ### PLOTS
    ################################################################################################
    
#    axis144 = [0, 5, 28, 48]
#    axis64 = [0, 5, 32, 52]
#    axis384 = [0, 5, 32, 52]
    axis144 = [0, 5, 25, 52]
    axis64 = [0, 5, 25, 52]
    axis384 = [0, 5, 25, 52]
        
    # plot conversation
    figure(1)
    key = calcKey("conversation", "fbir", 144, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-bo', label='FBIR 50 ms')
    hold(True)
    key = calcKey("conversation", "fbir", 144, 200)
    plot(data[key].getDataX(), data[key].getDataY(), '-gx', label='FBIR 200 ms')
    key = calcKey("conversation", "iplr", 144, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-r+', label='IPLR')

    axis(axis144)
    xlabel("Loss rate (%)")
    ylabel("Y-PSNR (dB)")
    title("Conversation, 144 kbit/s")
    legend()
    grid(True)
    hold(False)
    
    savefig("conversation_144kbps.eps")


    # plot foreman
    figure(2)
    key = calcKey("foreman", "fbir", 144, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-bo', label='FBIR 50 ms')
    hold(True)
    key = calcKey("foreman", "fbir", 144, 200)
    plot(data[key].getDataX(), data[key].getDataY(), '-gx', label='FBIR 200 ms')
    key = calcKey("foreman", "iplr", 144, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-r+', label='IPLR')

    axis(axis144)
    xlabel("Loss rate (%)")
    ylabel("Y-PSNR (dB)")
    title("Foreman, 144 kbit/s")
    legend()
    grid(True)
    hold(False)
    
    savefig("foreman_144kbps.eps")


    # plot soccer
    figure(3)
    key = calcKey("soccer", "fbir", 144, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-bo', label='FBIR 50 ms')
    hold(True)
    key = calcKey("soccer", "fbir", 144, 200)
    plot(data[key].getDataX(), data[key].getDataY(), '-gx', label='FBIR 200 ms')
    key = calcKey("soccer", "iplr", 144, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-r+', label='IPLR')


    axis(axis144)
    xlabel("Loss rate (%)")
    ylabel("Y-PSNR (dB)")
    title("Soccer, 144 kbit/s")
    legend()
    grid(True)
    hold(False)

    savefig("soccer_144kbps.eps")


    # plot conversation low rate
    figure(4)
    key = calcKey("conversation", "fbir", 64, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-bo', label='FBIR 50 ms')
    hold(True)
    key = calcKey("conversation", "fbir", 64, 200)
    plot(data[key].getDataX(), data[key].getDataY(), '-gx', label='FBIR 200 ms')
    key = calcKey("conversation", "iplr", 64, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-r+', label='IPLR')

    axis(axis64)
    xlabel("Loss rate (%)")
    ylabel("Y-PSNR (dB)")
    title("Conversation, 64 kbit/s")
    legend()
    grid(True)
    hold(False)

    savefig("conversation_064kbps.eps")

    # plot conversation high rate
    figure(5)
    key = calcKey("conversation", "fbir", 384, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-bo', label='FBIR 50 ms')
    hold(True)
    key = calcKey("conversation", "fbir", 384, 200)
    plot(data[key].getDataX(), data[key].getDataY(), '-gx', label='FBIR 200 ms')
    key = calcKey("conversation", "iplr", 384, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-r+', label='IPLR')

    axis(axis384)
    xlabel("Loss rate (%)")
    ylabel("Y-PSNR (dB)")
    title("Conversation, 384 kbit/s")
    legend()
    grid(True)
    hold(False)

    savefig("conversation_384kbps.eps")


    # plot all for conversation in one figure
    # plot conversation
    axisConv = [0, 5, 25, 52]
    figure(6)
    hold(True)
    grid(True)

    key = calcKey("conversation", "fbir", 144, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-bo', label='FBIR 50 ms')
    key = calcKey("conversation", "fbir", 144, 200)
    plot(data[key].getDataX(), data[key].getDataY(), '-gx', label='FBIR 200 ms')
    key = calcKey("conversation", "iplr", 144, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-r+', label='IPLR')

    key = calcKey("conversation", "fbir", 64, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '--bo', label='_nolegend_')
    key = calcKey("conversation", "fbir", 64, 200)
    plot(data[key].getDataX(), data[key].getDataY(), '--gx', label='_nolegend_')
    key = calcKey("conversation", "iplr", 64, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '--r+', label='_nolegend_')

    key = calcKey("conversation", "fbir", 384, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-.bo', label='_nolegend_')
    key = calcKey("conversation", "fbir", 384, 200)
    plot(data[key].getDataX(), data[key].getDataY(), '-.gx', label='_nolegend_')
    key = calcKey("conversation", "iplr", 384, 50)
    plot(data[key].getDataX(), data[key].getDataY(), '-.r+', label='_nolegend_')

    axis(axisConv)
    legend()
    xlabel("Loss rate (%)")
    ylabel("Y-PSNR (dB)")
    title("Conversation, various bit rates")

    savefig("conversation-combined.eps")


    # Show all figures
    #show()

if __name__ == "__main__":
    main()
