#!/usr/bin/env python

# Used to plot the temporal development of PSNR during 4000 frames.

FBIR="conv/fbir/144kbps/050ms/01plr"
IPLR="conv/iplr/144kbps/050ms/01plr"
FBIR2="conv/fbir2/144kbps/050ms/01plr"

avg_type = 1   # 1 = moving avg, 2 = block by block
window = 40
frames = 4000

import os, commands
import glob
import pylab

rootdir = os.getcwd()
psnrs = {}

for dir in [FBIR, FBIR2, IPLR]:
    # find files
    files = glob.glob(dir + "/" + "*vid.psnr")
    psnrs[dir] = []
    i =  0
    for fn in files:
        # read and store psnr values
        f =  file(fn, 'r')
        psnrs[dir].append([])
        for j in range(frames):
            psnrs[dir][i].append(float(f.readline()[-6:]))
        f.close()
        i = i + 1

if avg_type == 1:
    # moving average
    myrange = range(0, frames - window)
else:
    # block by block
    myrange = range(0, frames, window)


psnrs_avg = {}

for set in psnrs.keys():
    psnrs_avg[set] = []
    for i in myrange:
        means_window = []
        for psnrs_sequence in psnrs[set]:
            means_window.append(pylab.mean(psnrs_sequence[i:i+window]))
        psnrs_avg[set].append(pylab.mean(means_window))


pylab.grid(True)
pylab.hold(True)
pylab.plot(myrange, psnrs_avg[FBIR], 'b', label="FBIR 50 ms")
pylab.plot(myrange, psnrs_avg[FBIR2], '-.r', label="FBIR-2 50 ms")
pylab.plot(myrange, psnrs_avg[IPLR], '--g', label="IPLR") 

pylab.xlabel("Frame number")
pylab.ylabel("Y-PSNR (dB)")
pylab.title("Y-PSNR vs time: Conversation, 144 kbit/s, 1% loss")
pylab.legend()

#pylab.show()
pylab.savefig("psnr-time-long.eps")

