#include <pthread.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include "schedule.h"



/* The function waits until the time of execution of event E and calls the
 * appropriate function for this event. It frees E after execution.
 *
 * Should be called as a start routine for a thread.
 **/
static void *schedule2 (void *s)
{
    schedule_t *s2 = (schedule_t *) s;

    usleep(s2->usec);

    s2->callback(s2->args);

    // Free resources and exit thread.
    if (s2->args != NULL) free(s2->args);
    free (s2);
    
    pthread_exit(NULL);
}


/* Creates an event. The properties of this event should be passed as argument
 * S. S is a pointer to an allocated structure of type schedule_t. S will be
 * freed after execution of event.
 **/
void schedule (schedule_t *s)
{
    pthread_t thr;
    pthread_attr_t attr;
    int rc;

    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
    
    rc = pthread_create(&thr, NULL, schedule2, (void *) s);
    if (rc != 0)
      perror("Failed creating new thread");

    pthread_attr_destroy(&attr);
    //scheduleTask2 will free S.
}
