#include "defines.h"
#if IPLR

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <assert.h>

#include "global.h"
#include "image.h"
#include "iplr.h"

static int *iplr_counter; /* one counter for each MB */
static int iplr_totalNumberMB;

static FILE *iplr_log;


/*
 * Update log file
 */
static void iplr_log_update(Macroblock *mb)
{
  if (img->current_mb_nr == 0)
    fprintf(iplr_log, "\n%4d", img->number);
  fprintf(iplr_log, "\t%3d/%d (%d)", iplr_counter[img->current_mb_nr], iplr_intraflag[img->current_mb_nr], mb->mb_type);
}


/*
 * Init IPLR variables
 */
void iplr_init()
{
  int i;
  
  iplr_totalNumberMB = img->height * img->width / 256; /* # MBs */
  
  if ((iplr_counter = (int *) malloc(iplr_totalNumberMB * sizeof(int))) == NULL)
    no_mem_exit("iplr_init: counter");
  
  if ((iplr_intraflag = (int *) malloc(iplr_totalNumberMB * sizeof(int))) == NULL)
    no_mem_exit("iplr_init: intraflag");

  memset(iplr_counter, 0, iplr_totalNumberMB * sizeof(int));
  memset(iplr_intraflag, 0, iplr_totalNumberMB * sizeof(int));

  if(strlen(input->IPLRLogFile) > 0 && (iplr_log = fopen(input->IPLRLogFile, "wt")) == NULL) {
    snprintf(errortext, ET_SIZE, "Error open file %s", input->IPLRLogFile);
    error (errortext, 500);
  }
  
  fprintf(iplr_log, "IMAGE_NO / MB_NO\t\t Values: counter/intraflag (mb_type)\n\n");
  for (i = 0; i < iplr_totalNumberMB; i++)
    fprintf(iplr_log, "\t%3d", i);
}

/* Define which index hold what information */
#define IPLR_CONST_SKIP  0
#define IPLR_CONST_INTER 1
#define IPLR_CONST_INTRA 2
#define IPLR_CONST_LIMIT 3
/* The IPLR constants defining the behaviour. Only the default mode is tested. */
static const int iplr_const[][4] = { {-10, 1, 0, 1}, /* Default IPLR mode */
                                     {-10, 1, 0, 5} }; /* Less robust, more coding efficient. Untested! */

/*
 * Update IPLR variabels. Sets the iplr_intraflag[mb_number]
 */
void iplr_update(Macroblock *mb)
{
  assert(iplr_counter && iplr_intraflag);
  assert(img->current_mb_nr <= iplr_totalNumberMB);

  int mode = input->IPLRMode-1;

  iplr_intraflag[img->current_mb_nr] = 0;

  /* mode == skip && mv == 0 */
  if ((IS_COPY(mb) || IS_DIRECT(mb)) &&
      enc_picture->mv[LIST_0][img->block_y][img->block_x][0] == 0 &&
      enc_picture->mv[LIST_0][img->block_y][img->block_x][1] == 0)
    iplr_counter[img->current_mb_nr] = iplr_const[mode][IPLR_CONST_SKIP];
  
  /* == inter */ 
  else if (!IS_INTRA(mb)) 
    iplr_counter[img->current_mb_nr] += iplr_const[mode][IPLR_CONST_INTER];

  /* == intra */
  else
    iplr_counter[img->current_mb_nr] = iplr_const[mode][IPLR_CONST_INTRA];

  /* Set intra flag if counter is over limit */
  if (iplr_counter[img->current_mb_nr] > iplr_const[mode][IPLR_CONST_LIMIT]) {
    iplr_intraflag[img->current_mb_nr] = 1;
    iplr_counter[img->current_mb_nr] = 0;
  }

  iplr_log_update(mb);
}


/*
 * Free all variables
 */
void iplr_free()
{
  free(iplr_counter);
  free(iplr_intraflag);

  fprintf(iplr_log, "\n");
  fclose(iplr_log);
}

#endif /* IPLR */
